# -*- coding: utf-8 -*-
# pylint: skip-file

from alphacore import fit_linear
from alphacore import utils
from datetime import date

__title__ = 'alphacore'
__version__ = "0.0.5"
__description__ = 'Core statitstical functions for alpha'
__url__ = 'https://pypi.org/project/alphacore/'
__author__ = 'Matthew Reid'
__author_email__ = 'alpha.reliability@gmail.com'
__license__ = 'LGPLv3'
__copyright__ = "Copyright 2022-{}, Matthew Reid".format(date.today().year)
